﻿namespace EndMove.Sudoku.Solver.Core.Algorithms;

public class BackTrackingSolver : SolverBase
{
    public BackTrackingSolver(Action<int[,]> displayBoard) : base(displayBoard) { }

    public override (bool, int[,]) Solve(int[,] board)
    {
        int row = -1, col = -1;
        bool completed = true;
        for (int y = 0; y < 9; y++)
        {
            for (int x = 0; x < 9; x++)
            {
                if (board[y, x] != 0) continue;
                row = y;
                col = x;
                completed = false;
                break;
            }
            if (!completed) break;
        }

        if (completed)
        {
            return (true , board);
        }

        for (int n = 1; n <= 9; n++)
        {
            // Check if the wanted assignment is valid
            if (!IsValid(board, row, col, n)) continue;
            // Then assign display and continue solving
            board[row, col] = n;
            DisplayBoard(board);
            var (isSolved, solvedBoard) = Solve(board);
            // If the board is solved, return it
            if (isSolved) return (true, solvedBoard);
            // Otherwise, backtrack (reset the cell to a ready to work value)
            board[row, col] = 0;
        }
        return (false, board);
    }

    public override (bool, int[]) Solve(int[] board)
    {
        // TODO
        throw new NotImplementedException();
    }
}