﻿namespace EndMove.Sudoku.Solver.Core.Common;

public static class SudokuUtils
{
    /// <summary>
    /// Convert a 2D board to a 1D board
    /// </summary>
    /// <param name="board">2D Board</param>
    /// <param name="newBoard">1D Board</param>
    public static void TwoDTo1D(int[,] board, out int[] newBoard)
    {
        int rows = board.GetLength(0), cols = board.GetLength(1);
        newBoard = new int[rows * cols];
        for (int i = 0; i < rows; i++)
        {
            for (int j = 0; j < cols; j++)
            {
                newBoard[i * cols + j] = board[i, j];
            }
        }
    }

    /// Display the board in the terminal (1D Board)
    public static void DisplayBoard(int[] board)
    {
        const string separator = "+---+---+---+---+---+---+---+---+---+";
        for (int i = 0; i < 81; i++)
        {
            if (i % 27 == 0)
            {
                Console.WriteLine(separator);
            }
            Console.Write("{0} {1} ", (i % 3 == 0) ? '|' : ' ', board[i] == 0 ? "*" : board[i]);
            if (i % 9 == 8) Console.WriteLine("|");
        }
        Console.WriteLine(separator);
    }

    /// Display the board in the terminal (2D Board)
    public static void DisplayBoard(int[,] board)
    {
        const string separator = "+---+---+---+---+---+---+---+---+---+";
        Console.WriteLine(separator);
        for (int i = 0; i < 9; i++)
        {
            Console.Write("|");
            for (int j = 0; j < 9; j++)
            {
                Console.Write(" {0} {1}", board[i, j] == 0 ? "*" : board[i, j], ((j + 1) % 3 == 0) ? '|' : ' ');
            }
            Console.WriteLine();
            if ((i + 1) % 3 == 0)
            {
                Console.WriteLine(separator);
            }
        }
    }
}